﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using GroupLab.Networking;      // needed to use the shared dictionary
using System.Windows.Threading; // needed for UI threading

namespace HelloWorld
{
    /// <summary>
    /// A Groupware Hello World Program, where clicking a button says hello to all other program instances 
    /// </summary>
    public partial class MainWindow : Window
    {
        SharedDictionary shDict = new SharedDictionary(); // The shared dictionary we will connect to
        Subscription subscription = new Subscription();   // The data in the dictionary we will subscribe to
        const string KEY = "/hello";                      // The regular expression pattern we will set, get 
                                                          //  and subscribe to in the shared dictionary
        const string URL = "tcp://localhost:test";        // The URL describing the shared dictionary server location


        public MainWindow()
        {
            InitializeComponent();

            // Disabled the button until the dictionary is opened
            this.Dispatcher.Invoke(System.Windows.Threading.DispatcherPriority.Normal, new Action(delegate() 
            {
                button1.IsEnabled = false;
            }));

            // If a shared dictionary server already exists at the URL location, connect to it as a client 
            // Otherwise, this program will automatically become the server (but will also behave as a client as well)
            // Add an event handler to detect when the shared dictionary connecton is completely established
            this.shDict.Url = URL;
            this.shDict.Opened += new EventHandler(shDict_Opened); // Occurs when the connection to the shDictionary is opened after all subscriptions are updated
            this.shDict.Disconnected += new EventHandler<DisconnectedEventArgs>(shDict_Disconnected);

            // Initialize the subscription to the indicated key pattern. We put this inside a begin/end init to have it all processed together
            // The event handler is triggered when the key pattern is added or deleted in the shared dictionary, or its value is changed
            this.subscription.BeginInit();
            this.subscription.Dictionary = this.shDict;  // The dictionary associated with this subscription
            this.subscription.Pattern = KEY;         // Register the particular pattern we are interested in 
            this.subscription.Notified += new SubscriptionEventHandler(subscription_Notified);
            this.subscription.EndInit();

            this.shDict.Open();                          // Now open the shared dictionary
        }

        // Occurs when the primary connection to the shared dictionary server is opened,
        // and after initial subscription notifications have been invoked.
        void shDict_Opened(object sender, EventArgs e)
        {
            // Enable the button now that we are connected to the shared dictionary
            this.Dispatcher.Invoke(System.Windows.Threading.DispatcherPriority.Normal, new Action(delegate()
            {
                button1.IsEnabled = true;
            }));
        }

        // Occurs when the connection to the dictionary is lost
        void shDict_Disconnected(object sender, DisconnectedEventArgs e)
        {
            // Disabled the button until the dictionary is re-opened
            this.Dispatcher.Invoke(System.Windows.Threading.DispatcherPriority.Normal, new Action(delegate()
            {
                button1.IsEnabled = false;
            }));
        }

        // This event handler is called whenever the subsription pattern is altered
        void subscription_Notified(object sender, SubscriptionEventArgs e)
        {
            // get the value from the dictionary and set it as the button contents, 
            // although if that key was removed, just go back to a 'Click me' message. 
            // Reasons for the notification include Add, Changed, and Removed.
            string value = (e.Reason == SubscriptionNotification.Remove) ? "Click me" : (string)this.shDict.GetEntry(KEY).Value;
            this.Dispatcher.Invoke(System.Windows.Threading.DispatcherPriority.Normal, new Action(delegate()
            {
                button1.Content = value.ToString();
            }));
        }

        // Retrieve the name from the textbox, compose a message, and set the Key to that message.       
        private void button1_Click(object sender, RoutedEventArgs e)
        {
            this.shDict[KEY] = this.textBox1.Text + " says Hello!";
        }
    }
}
